@@ -27,9 +27,19 @@ namespace path_extend {
 class BidirectionalPath;
 
 struct Gap {
+    struct Trash {
+        uint32_t previous;
+        uint32_t current;
+
+        Trash(uint32_t previous_, uint32_t current_) :
+            previous(previous_), current(current_) {
+        }
+    };
     int gap;
-    uint32_t trash_previous;
-    uint32_t trash_current;
+    Trash trash;
+
+//True if gap is resolved not by ordinary procedure but by some sort of magic, and should not be changed.
+    bool is_final;
 
     static const int INVALID_GAP = std::numeric_limits<int>::min();
 
@@ -39,16 +49,21 @@ struct Gap {
     }
 
     //gap is in k+1-mers and does not know about "trash" regions
-    explicit Gap(int gap_ = 0, uint32_t trash_previous_ = 0, uint32_t trash_current_ = 0)
-     : gap(gap_), trash_previous(trash_previous_), trash_current(trash_current_)
+    explicit Gap(int gap_, Gap::Trash trash_, bool is_final_ = true)
+     : gap(gap_), trash(trash_), is_final(is_final_)
      { }
 
+    explicit Gap(int gap_ = 0, bool is_final_ = true)
+            : gap(gap_), trash{0, 0}, is_final(is_final_)
+    { }
+
     Gap conjugate() const {
-        return Gap(gap, trash_current, trash_previous);
+        return Gap(gap, {trash.current, trash.previous}, is_final);
     }
 
     bool operator==(const Gap &that) const {
-        return gap == that.gap && trash_previous == that.trash_previous && trash_current == that.trash_current;
+        return gap == that.gap && trash.previous == that.trash.previous && trash.current == that.trash.current
+               && is_final == that.is_final;
     }
 
     bool operator!=(const Gap &that) const {
@@ -60,16 +75,16 @@ struct Gap {
     }
 
     int overlap_after_trim(size_t k) const {
-        return overlap(k) - trash_current - trash_previous;
+        return overlap(k) - trash.current - trash.previous;
     }
 
     bool NoTrash() const {
-        return trash_current == 0 && trash_previous == 0;
+        return trash.current == 0 && trash.previous == 0;
     }
 };
 
 inline std::ostream& operator<<(std::ostream& os, Gap gap) {
-    return os << "[" << gap.gap << ", " << gap.trash_previous << ", " << gap.trash_current << "]";
+    return os << "[" << gap.gap << ", " << gap.trash.previous << ", " << gap.trash.current << "], final: "<< gap.is_final;
 }
 
 class PathListener {
@@ -104,9 +119,16 @@ public:
 
     BidirectionalPath(const Graph& g, const std::vector<EdgeId>& path)
             : BidirectionalPath(g) {
-        //TODO cumulative_len takes O(N^2) to fill
+        cumulative_len_.resize(path.size(), 0);
+        data_.resize(path.size());
+        gap_len_.resize(path.size(), Gap());
+
         for (size_t i = 0; i < path.size(); ++i) {
-            PushBack(path[i]);
+            data_[i] = path[i];
+            cumulative_len_.front() += g_.length(path[i]);
+        }
+        for (size_t i = 1; i < path.size(); ++i) {
+            cumulative_len_[i] = cumulative_len_[i - 1] - g_.length(data_[i - 1]);
         }
     }
 
@@ -523,9 +545,9 @@ private:
         data_.push_front(e);
         if (gap_len_.size() > 0) {
             VERIFY(gap_len_[0] == Gap());
-            gap_len_[0]= gap;
+            gap_len_[0] = gap;
         }
-        gap_len_.push_front(Gap(0, 0, 0));
+        gap_len_.push_front(Gap());
 
         int length = (int) g_.length(e);
         if (cumulative_len_.empty()) {
