@@ -14,6 +14,7 @@
 #pragma once
 
 #include "extension_chooser.hpp"
+#include "common/modules/alignment/gap_info.hpp"
 #include "assembly_graph/paths/bidirectional_path_container.hpp"
 #include "path_filter.hpp"
 #include "overlap_analysis.hpp"
@@ -241,10 +242,11 @@ public:
 
     void ResolveShortLoop(BidirectionalPath& path) const {
         EdgeId back_cycle_edge;
-        EdgeId loop_exit;
-        if (path.Size() >=1 && GetLoopAndExit(g_, path.Back(), back_cycle_edge, loop_exit)) {
+        EdgeId loop_outgoing;
+        EdgeId loop_incoming;
+        if (path.Size() >=1 && GetLoopAndExit(g_, path.Back(), back_cycle_edge, loop_outgoing, loop_incoming)) {
             DEBUG("Resolving short loop...");
-            MakeBestChoice(path, back_cycle_edge, loop_exit);
+            MakeBestChoice(path, back_cycle_edge, loop_outgoing, loop_incoming);
             DEBUG("Resolving short loop done");
         }
     }
@@ -254,42 +256,62 @@ private:
     const Graph& g_;
     shared_ptr<ShortLoopEstimator> loop_estimator_;
 
-    void UndoCycles(BidirectionalPath& p, EdgeId next_edge) const {
+    void UndoCycles(BidirectionalPath& p, EdgeId back_cycle_edge, EdgeId loop_incoming) const {
         if (p.Size() <= 2) {
             return;
         }
-        EdgeId first_edge = p.Back();
-        EdgeId second_edge = next_edge;
-        while (p.Size() > 2) {
-            if (p.At(p.Size() - 1) == first_edge && p.At(p.Size() - 2) == second_edge) {
-                p.PopBack(2);
+        EdgeId forward_cycle_edge = p.Back();
+        size_t loop_start_index = p.Size();
+        while (loop_start_index > 2) {
+            if (p.At(loop_start_index - 1) == forward_cycle_edge && p.At(loop_start_index - 2) == back_cycle_edge) {
+                loop_start_index -= 2;
             } else {
-                return;;
+                break;
             }
         }
+
+        if (p.At(loop_start_index - 1) == forward_cycle_edge) {
+            p.PopBack(p.Size() - loop_start_index);
+        } else if (loop_start_index != p.Size()) {
+            //Means we jumped into the loop
+            DEBUG("Jumped inside the loop, back loop edge " << g_.int_id(back_cycle_edge) << ", forward loop edge " << g_.int_id(forward_cycle_edge) << ", loop starts @ " << loop_start_index);
+            p.PrintDEBUG();
+            Gap gap = p.GapAt(loop_start_index);
+            p.PopBack(p.Size() - loop_start_index);
+
+            if (p.Back() != loop_incoming) {
+                p.PushBack(loop_incoming, Gap(max(0, gap.gap - (int) g_.length(loop_incoming)  - (int) g_.length(forward_cycle_edge)), {gap.trash.previous, 0}));
+            }
+            p.PushBack(forward_cycle_edge);
+            DEBUG("Restored the path");
+            p.PrintDEBUG();
+        }
     }
 
     //edges -- first edge is loop's back edge, second is loop exit edge
-    void MakeBestChoice(BidirectionalPath& path, EdgeId back_cycle_edge, EdgeId loop_exit) const {
+    void MakeBestChoice(BidirectionalPath& path, EdgeId back_cycle_edge, EdgeId loop_outgoing, EdgeId loop_incoming) const {
         EdgeId forward_cycle_edge = path.Back();
-        UndoCycles(path, back_cycle_edge);
+        UndoCycles(path, back_cycle_edge, loop_incoming);
 
         //Expects 0 (for no loops), 1 (for a single loop) or 2 (for many loops, will insert back_cycle_edge and Ns)
-        size_t loop_count = loop_estimator_->EstimateSimpleCycleCount(path, back_cycle_edge, loop_exit);
+        size_t loop_count = loop_estimator_->EstimateSimpleCycleCount(path, back_cycle_edge, loop_outgoing);
         if (loop_count > 0) {
             path.PushBack(back_cycle_edge);
             if (loop_count == 1) {
                 DEBUG("Single loop");
                 path.PushBack(forward_cycle_edge);
-                path.PushBack(loop_exit);
+                path.PushBack(loop_outgoing);
             }
             else {
                 DEBUG("Multiple cycles");
-                path.PushBack(loop_exit, Gap(int(g_.k() + BASIC_N_CNT)));
+                //If the forward edge is shorter than K, avoid overlapping bases between backward edge and outgoing edge
+                //Make sure that the N-stretch will be exactly 100 bp
+                uint32_t overlapping_bases = (uint32_t) max(int(g_.k()) - int(g_.length(forward_cycle_edge)), 0);
+                path.PushBack(loop_outgoing, Gap(int(g_.k() + BASIC_N_CNT - overlapping_bases), {0, overlapping_bases}));
             }
         }
         else {
-            path.PushBack(loop_exit);
+            path.PushBack(loop_outgoing);
         }
     }
 };
@@ -819,14 +841,11 @@ public:
 
     CompositeExtender(const Graph &g, GraphCoverageMap& cov_map,
                       UsedUniqueStorage &unique,
-                      const vector<shared_ptr<PathExtender>> &pes,
-                      size_t max_diff_len)
+                      const vector<shared_ptr<PathExtender>> &pes)
             : g_(g),
               cover_map_(cov_map),
               used_storage_(unique),
-              extenders_(pes),
-              max_diff_len_(max_diff_len) {
-    }
+              extenders_(pes) {}
 
     void GrowAll(PathContainer& paths, PathContainer& result) {
         result.clear();
@@ -844,7 +863,6 @@ private:
     GraphCoverageMap &cover_map_;
     UsedUniqueStorage &used_storage_;
     vector<shared_ptr<PathExtender>> extenders_;
-    size_t max_diff_len_;
 
     bool MakeGrowStep(BidirectionalPath& path, PathContainer* paths_storage) {
         DEBUG("make grow step composite extender");
@@ -925,7 +943,7 @@ protected:
         bool success = used_storage_.TryUseEdge(path, e, gap);
         if (success) {
             DEBUG("Adding edge. PathId: " << path.GetId() << " path length: " << path.Length() - 1 << ", fixed gap : "
-                                          << gap.gap << ", trash length: " << gap.trash_previous << "-" << gap.trash_current);
+                                          << gap.gap << ", trash length: " << gap.trash.previous << "-" << gap.trash.current);
         }
         return success;
     }
@@ -1160,21 +1178,8 @@ protected:
 
 
 class MultiExtender: public SimpleExtender {
-    size_t max_candidates_;
-
 public:
-    MultiExtender(const conj_graph_pack &gp,
-                  const GraphCoverageMap &cov_map,
-                  UsedUniqueStorage &unique,
-                  shared_ptr<ExtensionChooser> ec,
-                  size_t is,
-                  bool investigate_short_loops,
-                  bool use_short_loop_cov_resolver,
-                  double weight_threshold,
-                  size_t max_candidates = 0) :
-        SimpleExtender(gp, cov_map, unique, ec, is, investigate_short_loops, use_short_loop_cov_resolver, weight_threshold),
-        max_candidates_(max_candidates) {
-    }
+    using SimpleExtender::SimpleExtender;
 
 protected:
     bool AddCandidates(BidirectionalPath& path, PathContainer* paths_storage, ExtensionChooser::EdgeContainer& candidates) override {
@@ -1263,7 +1268,7 @@ class ScaffoldingPathExtender: public LoopDetectingPathExtender {
         }
         return Gap(gap.estimated_dist() + int(g_.k())
                    - int(gap.left_trim()) - int(gap.right_trim()),
-                   uint32_t(gap.left_trim()), uint32_t(gap.right_trim()));
+                   {uint32_t(gap.left_trim()), uint32_t(gap.right_trim())}, false);
     }
 
 protected:
@@ -1330,7 +1335,7 @@ protected:
         } else {
             DEBUG("Gap joiners off");
             VERIFY(candidates.back().d_ > int(g_.k()));
-            gap = Gap(candidates.back().d_);
+            gap = Gap(candidates.back().d_, false);
         }
 
         return TryUseEdge(path, e, NormalizeGap(gap));
@@ -1339,7 +1344,7 @@ protected:
     Gap NormalizeGap(Gap gap) const {
         VERIFY(gap != Gap::INVALID());
         if (gap.overlap_after_trim(g_.k()) > 0)
-            gap.trash_current += gap.overlap_after_trim(g_.k());
+            gap.trash.current += gap.overlap_after_trim(g_.k());
         return gap;
     }
 
