@@ -23,41 +23,43 @@
 
 using namespace llvm;
 
-namespace io {
-template<>
-void SequencingLibrary<debruijn_graph::config::DataSetData>::yamlize(llvm::yaml::IO &io) {
-  // First, load the "common stuff"
-  SequencingLibraryBase::yamlize(io);
-
-  // Now load the remaining stuff
-  io.mapOptional("read length"                , data_.read_length);
-  io.mapOptional("average read length"        , data_.avg_read_length);
-  io.mapOptional("insert size mean"           , data_.mean_insert_size);
-  io.mapOptional("insert size deviation"      , data_.insert_size_deviation);
-  io.mapOptional("insert size left quantile"  , data_.insert_size_left_quantile);
-  io.mapOptional("insert size right quantile" , data_.insert_size_right_quantile);
-  io.mapOptional("insert size median"         , data_.median_insert_size);
-  io.mapOptional("insert size mad"            , data_.insert_size_mad);
-  io.mapOptional("insert size distribution"   , data_.insert_size_distribution);
-  io.mapOptional("average coverage"           , data_.average_coverage);
-  io.mapOptional("pi threshold"               , data_.pi_threshold);
-  io.mapOptional("binary converted"           , data_.binary_reads_info.binary_coverted);
-  io.mapOptional("single reads mapped"        , data_.single_reads_mapped);
-  io.mapOptional("library index"              , data_.lib_index);
-  io.mapOptional("number of reads"            , data_.read_count);
-  io.mapOptional("total nucleotides"          , data_.total_nucls);
-}
-
-template<>
-void SequencingLibrary<debruijn_graph::config::DataSetData>::validate(llvm::yaml::IO &io, llvm::StringRef &res) {
-  // Simply ask base class to validate for us
-  SequencingLibraryBase::validate(io, res);
-}
-}
-
 #include "pipeline/library.inl"
 
-template class io::DataSet<debruijn_graph::config::DataSetData>;
+namespace llvm { namespace yaml {
+
+template<> struct MappingTraits<debruijn_graph::config::dataset> {
+    static void mapping(IO& io, debruijn_graph::config::dataset& cfg) {
+        io.mapRequired("max read length", cfg.RL);
+        io.mapRequired("nomerge max read length", cfg.no_merge_RL);
+        io.mapRequired("average read length", cfg.aRL);
+        io.mapRequired("average coverage", cfg.average_coverage);
+        io.mapRequired("libraries", cfg.reads);
+    }
+};
+
+template<> struct MappingTraits<debruijn_graph::config::LibraryData> {
+    static void mapping(IO& io, debruijn_graph::config::LibraryData& data) {
+        io.mapRequired("unmerged read length"       , data.unmerged_read_length);
+        io.mapRequired("merged read length"         , data.merged_read_length);
+        io.mapRequired("insert size mean"           , data.mean_insert_size);
+        io.mapRequired("insert size deviation"      , data.insert_size_deviation);
+        io.mapRequired("insert size left quantile"  , data.insert_size_left_quantile);
+        io.mapRequired("insert size right quantile" , data.insert_size_right_quantile);
+        io.mapRequired("insert size median"         , data.median_insert_size);
+        io.mapRequired("insert size mad"            , data.insert_size_mad);
+        io.mapRequired("insert size distribution"   , data.insert_size_distribution);
+        io.mapRequired("pi threshold"               , data.pi_threshold);
+        io.mapRequired("binary converted"           , data.binary_reads_info.binary_converted);
+        io.mapRequired("single reads mapped"        , data.single_reads_mapped);
+        io.mapRequired("library index"              , data.lib_index);
+        io.mapRequired("number of reads"            , data.read_count);
+        io.mapRequired("total nucleotides"          , data.total_nucls);
+    }
+};
+
+} }
+
+template class io::DataSet<debruijn_graph::config::LibraryData>;
 
 namespace debruijn_graph {
 namespace config {
@@ -76,6 +78,7 @@ vector<string> CheckedNames(const vector<pair<string, mode_t>>& mapping, mode_t
 vector<string> InfoPrinterPosNames() {
     return CheckedNames<info_printer_pos>({
                     {"default", info_printer_pos::default_pos},
+                    {"before_raw_simplification", info_printer_pos::before_raw_simplification},
                     {"before_first_gap_closer", info_printer_pos::before_first_gap_closer},
                     {"before_simplification", info_printer_pos::before_simplification},
                     {"before_post_simplification", info_printer_pos::before_post_simplification},
@@ -104,14 +107,6 @@ vector<string> ConstructionModeNames() {
                     {"extension", construction_mode::extention}}, construction_mode::total);
 }
 
-vector<string> EstimationModeNames() {
-    return CheckedNames<estimation_mode>({
-                    {"simple", estimation_mode::simple},
-                    {"weighted", estimation_mode::weighted},
-                    {"smoothing", estimation_mode::smoothing}}, estimation_mode::total);
-}
-
-
 vector<string> ResolveModeNames() {
     return CheckedNames<resolving_mode>({
                     {"none", resolving_mode::none},
@@ -132,45 +127,42 @@ vector<string> BrokenScaffoldsModeNames() {
                                              {"break_all", output_broken_scaffolds::break_all}}, output_broken_scaffolds::total);
 }
 
+template<class T>
+void LoadFromYaml(const std::string& filename, T &t) {
+    auto buf = llvm::MemoryBuffer::getFile(filename);
+    VERIFY_MSG(buf, "Failed to load file " << filename);
+    llvm::yaml::Input yin(*buf.get());
+    VERIFY_MSG(!yin.error(), "Failed to load file " << filename);
+    yin >> t;
+}
+
+template<class T>
+void WriteToYaml(T &t, const std::string& filename) {
+    std::error_code EC;
+    llvm::raw_fd_ostream ofs(filename, EC, llvm::sys::fs::OpenFlags::F_Text);
+    llvm::yaml::Output yout(ofs);
+    yout << t;
+}
 
 void load_lib_data(const std::string& prefix) {
-  // First, load the data into separate libs
-  cfg::get_writable().ds.reads.load(prefix + ".lib_data");
-
-  // Now, infer the common parameters
-  size_t max_rl = 0;
-  double avg_cov = 0.0;
-  double avg_rl = 0.0;
-  for (const auto& lib : cfg::get().ds.reads.libraries()) {
-      auto const& data = lib.data();
-      if (lib.is_graph_contructable())
-          max_rl = std::max(max_rl, data.read_length);
-      if (data.average_coverage > 0)
-          avg_cov = data.average_coverage;
-      if (data.avg_read_length > 0)
-          avg_rl = data.avg_read_length;
-  }
-
-  cfg::get_writable().ds.set_RL(max_rl);
-  cfg::get_writable().ds.set_aRL(avg_rl);
-  cfg::get_writable().ds.set_avg_coverage(avg_cov);
+    LoadFromYaml(prefix + ".lib_data", cfg::get_writable().ds);
 }
 
 void write_lib_data(const std::string& prefix) {
-  cfg::get_writable().ds.reads.save(prefix + ".lib_data");
+    WriteToYaml(cfg::get_writable().ds, prefix + ".lib_data");
 }
 
 void load(debruijn_config::simplification::tip_clipper &tc,
-          boost::property_tree::ptree const &pt, bool /*complete*/) {
-     using config_common::load;
-     load(tc.condition, pt, "condition");
+          boost::property_tree::ptree const &pt, bool complete) {
+    using config_common::load;
+    load(tc.condition, pt, "condition", complete);
 }
 
 void load(debruijn_config::simplification::dead_end_clipper& dead_end,
           boost::property_tree::ptree const &pt,
-          bool /* complete */) {
+          bool complete) {
     using config_common::load;
-    load(dead_end.condition, pt, "condition");
+    load(dead_end.condition, pt, "condition", complete);
     load(dead_end.enabled, pt, "enabled");
 }
 
@@ -205,7 +197,7 @@ void load(construction_mode& con_mode,
 }
 
 void load(debruijn_config::construction::early_tip_clipper& etc,
-          boost::property_tree::ptree const& pt, bool /*complete*/) {
+          boost::property_tree::ptree const& pt, bool) {
     using config_common::load;
     load(etc.enable, pt, "enable");
     etc.length_bound = pt.get_optional<size_t>("length_bound");
@@ -217,29 +209,17 @@ void load(debruijn_config::construction& con,
     load(con.con_mode, pt, "mode", complete);
     load(con.keep_perfect_loops, pt, "keep_perfect_loops", complete);
     load(con.read_buffer_size, pt, "read_buffer_size", complete);
+    load(con.read_cov_threshold, pt, "read_cov_threshold", complete);
+
     con.read_buffer_size *= 1024 * 1024;
     load(con.early_tc, pt, "early_tip_clipper", complete);
 }
 
-void load(debruijn_config::sensitive_mapper& sensitive_map,
-          boost::property_tree::ptree const& pt, bool complete) {
-    using config_common::load;
-    load(sensitive_map.k, pt, "k", complete);
-}
-
-void load(estimation_mode &est_mode,
-          boost::property_tree::ptree const &pt, std::string const &key,
-          bool complete) {
-    if (complete || pt.find(key) != pt.not_found()) {
-        est_mode = ModeByName<estimation_mode>(pt.get<std::string>(key), EstimationModeNames());
-    }
-}
-
 void load(debruijn_config::simplification::bulge_remover& br,
           boost::property_tree::ptree const& pt, bool complete) {
   using config_common::load;
 
-  load(br.enabled                           , pt,   "enabled"					  , complete);
+  load(br.enabled                           , pt,   "enabled");
   load(br.main_iteration_only               , pt,   "main_iteration_only"	      , complete);
   load(br.max_bulge_length_coefficient		, pt,   "max_bulge_length_coefficient", complete);
   load(br.max_additive_length_coefficient	, pt,
@@ -256,18 +236,10 @@ void load(debruijn_config::simplification::bulge_remover& br,
   load(br.buff_cov_rel_diff,                pt,     "buff_cov_rel_diff", complete);
 }
 
-void load(debruijn_config::simplification::topology_tip_clipper& ttc,
-          boost::property_tree::ptree const& pt, bool /*complete*/) {
-  using config_common::load;
-  load(ttc.length_coeff, pt, "length_coeff");
-  load(ttc.plausibility_length, pt, "plausibility_length");
-  load(ttc.uniqueness_length, pt, "uniqueness_length");
-}
-
 void load(debruijn_config::simplification::complex_tip_clipper &ctc,
           boost::property_tree::ptree const &pt, bool complete) {
     using config_common::load;
-    load(ctc.enabled, pt, "enabled", complete);
+    load(ctc.enabled, pt, "enabled");
     load(ctc.max_relative_coverage, pt, "max_relative_coverage", complete);
     load(ctc.max_edge_len, pt, "max_edge_len", complete);
     load(ctc.condition, pt, "condition", complete);
@@ -276,15 +248,16 @@ void load(debruijn_config::simplification::complex_tip_clipper &ctc,
 void load(debruijn_config::simplification::relative_coverage_edge_disconnector& relative_ed,
         boost::property_tree::ptree const& pt, bool complete) {
   using config_common::load;
-  load(relative_ed.enabled, pt, "enabled", complete);
+  load(relative_ed.enabled, pt, "enabled");
   load(relative_ed.diff_mult, pt, "diff_mult", complete);
   load(relative_ed.edge_sum, pt, "edge_sum", complete);
+  load(relative_ed.unconditional_diff_mult, pt, "unconditional_diff_mult", complete);
 }
 
 void load(debruijn_config::simplification::relative_coverage_comp_remover& rcc,
           boost::property_tree::ptree const& pt, bool complete) {
   using config_common::load;
-  load(rcc.enabled, pt, "enabled", complete);
+  load(rcc.enabled, pt, "enabled");
   load(rcc.coverage_gap, pt, "coverage_gap", complete);
   load(rcc.length_coeff, pt, "max_length_coeff", complete);
   load(rcc.tip_allowing_length_coeff, pt, "max_length_with_tips_coeff", complete);
@@ -296,7 +269,7 @@ void load(debruijn_config::simplification::relative_coverage_comp_remover& rcc,
 void load(debruijn_config::simplification::isolated_edge_remover& ier,
           boost::property_tree::ptree const& pt, bool complete) {
   using config_common::load;
-  load(ier.enabled, pt, "enabled", complete);
+  load(ier.enabled, pt, "enabled");
   load(ier.max_length, pt, "max_length", complete);
   load(ier.use_rl_for_max_length, pt, "use_rl_for_max_length", complete);
   load(ier.max_coverage, pt, "max_coverage", complete);
@@ -307,7 +280,7 @@ void load(debruijn_config::simplification::isolated_edge_remover& ier,
 void load(debruijn_config::simplification::low_covered_edge_remover& lcer,
           boost::property_tree::ptree const& pt, bool complete) {
   using config_common::load;
-  load(lcer.enabled, pt, "lcer_enabled", complete);
+  load(lcer.enabled, pt, "lcer_enabled");
   load(lcer.coverage_threshold, pt, "lcer_coverage_threshold", complete);
 }
 
@@ -340,12 +313,12 @@ void load(debruijn_config::simplification::erroneous_connections_remover& ec,
 }
 
 void load(debruijn_config::simplification::relative_coverage_ec_remover& rcec,
-          boost::property_tree::ptree const& pt, bool /*complete*/) {
+          boost::property_tree::ptree const& pt, bool complete) {
     using config_common::load;
 
     load(rcec.enabled, pt, "enabled");
-    load(rcec.max_ec_length, pt, "rcec_lb");
-    load(rcec.rcec_ratio, pt, "rcec_cb");
+    load(rcec.max_ec_length, pt, "rcec_lb", complete);
+    load(rcec.rcec_ratio, pt, "rcec_cb", complete);
 }
 
 void load(debruijn_config::simplification::topology_based_ec_remover& tec,
@@ -383,6 +356,14 @@ void load(debruijn_config::simplification::max_flow_ec_remover& mfec,
   load(mfec.uniqueness_length, pt, "uniqueness_length");
 }
 
+void load(debruijn_config::simplification::topology_tip_clipper& ttc,
+          boost::property_tree::ptree const& pt, bool complete) {
+  using config_common::load;
+  load(ttc.length_coeff, pt, "length_coeff", complete);
+  load(ttc.plausibility_length, pt, "plausibility_length", complete);
+  load(ttc.uniqueness_length, pt, "uniqueness_length", complete);
+}
+
 void load(debruijn_config::simplification::hidden_ec_remover& her,
           boost::property_tree::ptree const& pt, bool /*complete*/) {
   using config_common::load;
@@ -425,7 +406,7 @@ void load(debruijn_config::ambiguous_distance_estimator &amde,
           boost::property_tree::ptree const &pt, bool complete) {
     using config_common::load;
 
-    load(amde.enabled, pt, "enabled", complete);
+    load(amde.enabled, pt, "enabled");
     load(amde.haplom_threshold, pt, "haplom_threshold", complete);
     load(amde.relative_length_threshold, pt, "relative_length_threshold", complete);
     load(amde.relative_seq_threshold, pt, "relative_seq_threshold", complete);
@@ -447,35 +428,28 @@ void load(debruijn_config::truseq_analysis& tsa,
   load(tsa.genome_file, pt, "genome_file");
 }
 
-void load(debruijn_config::bwa_aligner& bwa,
+void load(bwa_aligner& bwa,
           boost::property_tree::ptree const& pt, bool /*complete*/) {
     using config_common::load;
-    load(bwa.bwa_enable, pt, "bwa_enable");
-    load(bwa.debug, pt, "debug");
-    load(bwa.path_to_bwa, pt, "path_to_bwa");
     load(bwa.min_contig_len, pt, "min_contig_len");
 }
 
-void load(debruijn_config::pacbio_processor& pb,
+void load(pacbio_processor& pb,
           boost::property_tree::ptree const& pt, bool /*complete*/) {
   using config_common::load;
-  load(pb.pacbio_k, pt, "pacbio_k");
-  load(pb.additional_debug_info, pt, "additional_debug_info");
+
+  load(pb.bwa_length_cutoff, pt, "bwa_length_cutoff");
   load(pb.compression_cutoff, pt, "compression_cutoff");
-  load(pb.domination_cutoff, pt, "domination_cutoff");
   load(pb.path_limit_stretching, pt, "path_limit_stretching");
   load(pb.path_limit_pressing, pt, "path_limit_pressing");
   load(pb.max_path_in_dijkstra, pt, "max_path_in_dijkstra");
   load(pb.max_vertex_in_dijkstra, pt, "max_vertex_in_dijkstra");
-  load(pb.ignore_middle_alignment, pt, "ignore_middle_alignment");
   load(pb.long_seq_limit, pt, "long_seq_limit");
   load(pb.pacbio_min_gap_quantity, pt, "pacbio_min_gap_quantity");
   load(pb.contigs_min_gap_quantity, pt, "contigs_min_gap_quantity");
   load(pb.max_contigs_gap_length, pt, "max_contigs_gap_length");
-
 }
 
-
 void load(debruijn_config::position_handler& pos,
           boost::property_tree::ptree const& pt, bool /*complete*/) {
   using config_common::load;
@@ -486,6 +460,7 @@ void load(debruijn_config::position_handler& pos,
   load(pos.late_threading, pt, "late_threading");
   load(pos.careful_labeling, pt, "careful_labeling");
 }
+
 void load(debruijn_config::plasmid& pd,
           boost::property_tree::ptree const& pt, bool /*complete*/) {
     using config_common::load;
@@ -497,16 +472,14 @@ void load(debruijn_config::plasmid& pd,
     load(pd.small_component_relative_coverage, pt, "small_component_relative_coverage");
     load(pd.min_component_length, pt, "min_component_length");
     load(pd.min_isolated_length, pt, "min_isolated_length");
-
 }
 
-
 void load(debruijn_config::gap_closer& gc,
           boost::property_tree::ptree const& pt, bool /*complete*/) {
   using config_common::load;
   load(gc.minimal_intersection, pt, "minimal_intersection");
+  load(gc.before_raw_simplify, pt, "before_raw_simplify");
   load(gc.before_simplify, pt, "before_simplify");
-  load(gc.in_simplify, pt, "in_simplify");
   load(gc.after_simplify, pt, "after_simplify");
   load(gc.weight_threshold, pt, "weight_threshold");
 }
@@ -544,44 +517,39 @@ void load(debruijn_config::kmer_coverage_model& kcm,
 }
 
 void load(dataset &ds,
-          boost::property_tree::ptree const &pt, bool /*complete*/) {
+          boost::property_tree::ptree const &pt,
+          const std::string &input_dir) {
     using config_common::load;
 
-  load(ds.reads_filename, pt, "reads");
+    //loading reads
+    std::string reads_filename;
+    load(reads_filename, pt, "reads");
 
-    ds.reference_genome_filename = "";
+    if (reads_filename[0] != '/')
+        reads_filename = input_dir + reads_filename;
+    fs::CheckFileExistenceFATAL(reads_filename);
+    ds.reads.load(reads_filename);
+
+    //loading reference
+    std::string reference_genome_filename;
     boost::optional<std::string> refgen =
             pt.get_optional<std::string>("reference_genome");
     if (refgen && *refgen != "N/A") {
-        ds.reference_genome_filename = *refgen;
+        reference_genome_filename = *refgen;
     }
-}
 
-void load_reads(dataset &ds,
-                std::string input_dir) {
-    if (ds.reads_filename[0] != '/')
-        ds.reads_filename = input_dir + ds.reads_filename;
-    fs::CheckFileExistenceFATAL(ds.reads_filename);
-    ds.reads.load(ds.reads_filename);
-}
-
-void load_reference_genome(dataset &ds,
-                           std::string input_dir) {
-    if (ds.reference_genome_filename == "") {
-        ds.reference_genome = vector<std::string>(0);
+    if (reference_genome_filename == "")
         return;
-    }
-    if (ds.reference_genome_filename[0] != '/')
-        ds.reference_genome_filename = input_dir + ds.reference_genome_filename;
-    fs::CheckFileExistenceFATAL(ds.reference_genome_filename);
-    io::FileReadStream genome_stream(ds.reference_genome_filename);
+
+    if (reference_genome_filename[0] != '/')
+        reference_genome_filename = input_dir + reference_genome_filename;
+    fs::CheckFileExistenceFATAL(reference_genome_filename);
+    io::FileReadStream genome_stream(reference_genome_filename);
     while (!genome_stream.eof()) {
         io::SingleRead genome;
         genome_stream >> genome;
         ds.reference_genome.push_back(genome.GetSequenceString());
     }
-
-
 }
 
 void load(debruijn_config::simplification& simp,
@@ -590,7 +558,6 @@ void load(debruijn_config::simplification& simp,
 
   load(simp.cycle_iter_count, pt, "cycle_iter_count", complete);
 
-  load(simp.post_simplif_enabled, pt, "post_simplif_enabled", complete);
   load(simp.topology_simplif_enabled, pt, "topology_simplif_enabled", complete);
   load(simp.tc, pt, "tc", complete); // tip clipper:
 
@@ -601,7 +568,7 @@ void load(debruijn_config::simplification& simp,
   load(simp.ec, pt, "ec", complete); // erroneous connections remover:
   load(simp.rcec, pt, "rcec", complete); // relative coverage erroneous connections remover
   load(simp.rcc, pt, "rcc", complete); // relative coverage component remover:
-  load(simp.relative_ed, pt, "relative_ed", complete); // relative edge disconnector:
+  load(simp.red, pt, "red", complete); // relative edge disconnector:
   load(simp.tec, pt, "tec", complete); // topology aware erroneous connections remover:
   load(simp.trec, pt, "trec", complete); // topology and reliability based erroneous connections remover:
   load(simp.isec, pt, "isec", complete); // interstrand erroneous connections remover (thorn remover):
@@ -714,19 +681,14 @@ void load_launch_info(debruijn_config &cfg, boost::property_tree::ptree const &p
         cfg.temp_bin_reads_dir += '/';
 
     load(cfg.max_threads, pt, "max_threads");
-    // Fix number of threads according to OMP capabilities.
-    cfg.max_threads = std::min(cfg.max_threads, (size_t) omp_get_max_threads());
-    // Inform OpenMP runtime about this :)
-    omp_set_num_threads((int) cfg.max_threads);
+    cfg.max_threads = spades_set_omp_threads(cfg.max_threads);
 
     load(cfg.max_memory, pt, "max_memory");
 
     fs::CheckFileExistenceFATAL(cfg.dataset_file);
     boost::property_tree::ptree ds_pt;
     boost::property_tree::read_info(cfg.dataset_file, ds_pt);
-    load(cfg.ds, ds_pt, true);
-    load_reads(cfg.ds, cfg.input_dir);
-    load_reference_genome(cfg.ds, cfg.input_dir);
+    load(cfg.ds, ds_pt, cfg.input_dir);
 }
 
 // main debruijn config load function
@@ -759,7 +721,6 @@ void load_cfg(debruijn_config &cfg, boost::property_tree::ptree const &pt,
 
     load(cfg.max_repeat_length, pt, "max_repeat_length", complete);
 
-    load(cfg.est_mode, pt, "estimation_mode", complete);
     load(cfg.de, pt, "de", complete);
     load(cfg.ade, pt, "ade", complete); // advanced distance estimator:
     load(cfg.amb_de, pt, "amb_de", complete);
@@ -780,8 +741,6 @@ void load_cfg(debruijn_config &cfg, boost::property_tree::ptree const &pt,
     load(cfg.use_scaffolder, pt, "use_scaffolder", complete);
     load(cfg.avoid_rc_connections, pt, "avoid_rc_connections", complete);
 
-    load(cfg.sensitive_map, pt, "sensitive_mapper", complete);
-
     bool save_gp;
     load(save_gp, pt, "save_gp", complete);
     load(cfg.info_printers, pt, "info_printers", complete);
@@ -827,6 +786,21 @@ void load(debruijn_config &cfg, const std::string &cfg_fns) {
     load(cfg, std::vector<std::string>({ cfg_fns }));
 }
 
+void init_libs(io::DataSet<LibraryData> &dataset, size_t max_threads,
+               size_t buffer_size, const std::string &temp_bin_reads_path) {
+    for (size_t i = 0; i < dataset.lib_count(); ++i) {
+        auto& lib = dataset[i];
+        lib.data().lib_index = i;
+        auto& bin_info = lib.data().binary_reads_info;
+        bin_info.chunk_num = max_threads;
+        bin_info.buffer_size = buffer_size;
+        bin_info.bin_reads_info_file = temp_bin_reads_path + "INFO_" + std::to_string(i);
+        bin_info.paired_read_prefix = temp_bin_reads_path + "paired_" + std::to_string(i);
+        bin_info.merged_read_prefix = temp_bin_reads_path + "merged_" + std::to_string(i);
+        bin_info.single_read_prefix = temp_bin_reads_path + "single_" + std::to_string(i);
+    }
+}
+
 void load(debruijn_config &cfg, const std::vector<std::string> &cfg_fns) {
     VERIFY_MSG(cfg_fns.size() > 0, "Should provide at least one config file");
     boost::property_tree::ptree base_pt;
@@ -873,15 +847,7 @@ void load(debruijn_config &cfg, const std::vector<std::string> &cfg_fns) {
              + cfg.temp_bin_reads_dir);
     //cfg.temp_bin_reads_info = cfg.temp_bin_reads_path + "INFO";
 
-    for (size_t i = 0; i < cfg.ds.reads.lib_count(); ++i) {
-        auto& lib = cfg.ds.reads[i];
-        lib.data().lib_index = i;
-        lib.data().binary_reads_info.chunk_num = cfg.max_threads;
-        lib.data().binary_reads_info.bin_reads_info_file = cfg.temp_bin_reads_path + "INFO_" + std::to_string(i);
-        lib.data().binary_reads_info.buffer_size = cfg.buffer_size;
-        lib.data().binary_reads_info.paired_read_prefix = cfg.temp_bin_reads_path + "paired_" + std::to_string(i);
-        lib.data().binary_reads_info.single_read_prefix = cfg.temp_bin_reads_path + "single_" + std::to_string(i);
-    }
+    init_libs(cfg.ds.reads, cfg.max_threads, cfg.buffer_size, cfg.temp_bin_reads_path);
 }
 
 }
