@@ -46,12 +46,6 @@ namespace simplification {
 //todo remove this line
 using namespace debruijn_graph;
 
-template<class Graph>
-using AlgoPtr = std::shared_ptr<omnigraph::PersistentAlgorithmBase<Graph>>;
-
-template<class Graph>
-using EdgeConditionT = func::TypedPredicate<typename Graph::EdgeId>;
-
 template<class Graph>
 class ConditionParser {
 private:
@@ -61,12 +55,13 @@ private:
     string next_token_;
     string input_;
     const SimplifInfoContainer settings_;
-    size_t curr_iteration_;
-    size_t iteration_cnt_;
+    /*(curr_iter + 1) / total_iter*/
+    double iter_run_progress_;
     std::queue<string> tokenized_input_;
 
     size_t max_length_bound_;
     double max_coverage_bound_;
+    int requested_iterations_;
 
     string ReadNext() {
         if (!tokenized_input_.empty()) {
@@ -122,12 +117,21 @@ private:
         } else if (next_token_ == "rlmk") {
             //Read length minus k
             VERIFY_MSG(settings_.read_length() > g_.k(), "Read length was shorter than K");
-            DEBUG("Creating (rl - k) bound");
-            size_t length_bound = settings_.read_length() - g_.k();
+            double length_coeff = std::stod(ReadNext());
+            DEBUG("Creating (rl - k) bound. Multiplicative coefficient: " << length_coeff);
+            size_t length_bound = size_t(math::round(length_coeff * double(settings_.read_length() - g_.k())));
+            RelaxMin(min_length_bound, length_bound);
+            DEBUG("Min length bound - " << min_length_bound);
+            return LengthUpperBound<Graph>(g_, length_bound);
+        } else if (next_token_ == "rl") {
+            //Read length
+            VERIFY_MSG(settings_.read_length() > 0, "Read was zero");
+            double length_coeff = std::stod(ReadNext());
+            DEBUG("Creating rl bound. Multiplicative coefficient: " << length_coeff);
+            size_t length_bound = size_t(math::round(length_coeff * double(settings_.read_length())));
             RelaxMin(min_length_bound, length_bound);
             DEBUG("Min length bound - " << min_length_bound);
             return LengthUpperBound<Graph>(g_, length_bound);
-
         } else if (next_token_ == "to_ec_lb") {
             double length_coeff = std::stod(ReadNext());
 
@@ -170,10 +174,9 @@ private:
             RelaxMin(min_coverage_bound, cov_bound);
             return CoverageUpperBound<Graph>(g_, cov_bound);
         } else if (next_token_ == "icb") {
-            VERIFY(iteration_cnt_ != -1ul && curr_iteration_ != -1ul);
             ReadNext();
             double cov_bound = GetCoverageBound();
-            cov_bound = cov_bound / (double) iteration_cnt_ * (double) (curr_iteration_ + 1);
+            cov_bound = cov_bound * iter_run_progress_;
             DEBUG("Creating iterative coverage upper bound " << cov_bound);
             RelaxMin(min_coverage_bound, cov_bound);
             return CoverageUpperBound<Graph>(g_, cov_bound);
@@ -189,7 +192,7 @@ private:
         } else if (next_token_ == "mmm") {
             ReadNext();
             DEBUG("Creating max mismatches cond " << next_token_);
-            return MismatchTipCondition<Graph>(g_, std::stoll(next_token_));
+            return MismatchTipCondition<Graph>(g_, std::stod(next_token_));
         } else {
             VERIFY(false);
             return func::AlwaysTrue<EdgeId>();
@@ -212,21 +215,22 @@ private:
 public:
 
     ConditionParser(const Graph &g, string input, const SimplifInfoContainer &settings,
-                    size_t curr_iteration = -1ul, size_t iteration_cnt = -1ul)
+                    double iter_run_progress = 1.)
+                    //size_t curr_iteration = 0, size_t iteration_cnt = 1)
             : g_(g),
               input_(input),
               settings_(settings),
-              curr_iteration_(curr_iteration),
-              iteration_cnt_(iteration_cnt),
+              iter_run_progress_(iter_run_progress),
+              //iter_run_progress_((double) (curr_iteration + 1) / (double) iteration_cnt),
               max_length_bound_(0),
-              max_coverage_bound_(0.) {
+              max_coverage_bound_(0.),
+              requested_iterations_(1) {
         DEBUG("Creating parser for string " << input);
         using namespace boost;
         vector<string> tmp_tokenized_input;
         boost::split(tmp_tokenized_input, input_, boost::is_any_of(" ,;"), boost::token_compress_on);
-        for (auto it = tmp_tokenized_input.begin();
-             it != tmp_tokenized_input.end(); ++it) {
-            tokenized_input_.push(*it);
+        for (const auto &s : tmp_tokenized_input) {
+            tokenized_input_.push(s);
         }
         ReadNext();
     }
@@ -234,6 +238,11 @@ public:
     func::TypedPredicate<EdgeId> operator()() {
         DEBUG("Parsing");
         func::TypedPredicate<EdgeId> answer = func::AlwaysFalse<EdgeId>();
+        if (next_token_ == "loop") {
+            requested_iterations_ = std::stoi(ReadNext());
+            ReadNext();
+        }
+
         VERIFY_MSG(next_token_ == "{", "Expected \"{\", but next token was " << next_token_);
         while (next_token_ == "{") {
             size_t min_length_bound = numeric_limits<size_t>::max();
@@ -255,6 +264,10 @@ public:
         return max_coverage_bound_;
     }
 
+    int requested_iterations() const {
+        return requested_iterations_;
+    }
+
 private:
     DECL_LOGGER("ConditionParser");
 };
@@ -270,12 +283,7 @@ public:
     }
 
     bool operator()(EdgeId edge, const vector<EdgeId>& path) const {
-        vector<Sequence> path_sequences;
-        for (EdgeId e : path) {
-            path_sequences.push_back(g_.EdgeNucls(e));
-        }
-        Sequence path_sequence(
-            MergeOverlappingSequences(path_sequences, g_.k()));
+        Sequence path_sequence = MergeSequences(g_, path);
         size_t dist = EditDistance(g_.EdgeNucls(edge), path_sequence);
         TRACE( "Bulge sequences with distance " << dist << " were " << g_.EdgeNucls(edge) << " and " << path_sequence);
         return true;
@@ -293,11 +301,10 @@ EdgeRemovalHandlerF<typename gp_t::graph_t> WrapWithProjectionCallback(
     typedef typename gp_t::graph_t Graph;
     typedef typename Graph::EdgeId EdgeId;
     TipsProjector<gp_t> tip_projector(gp);
-
-    EdgeRemovalHandlerF<Graph> projecting_callback = std::bind(&TipsProjector<gp_t>::ProjectTip,
-                                             tip_projector, std::placeholders::_1);
-
-    return func::CombineCallbacks<EdgeId>(std::ref(removal_handler), projecting_callback);
+    return [=](EdgeId e) {
+        if (removal_handler) removal_handler(e);
+        tip_projector.ProjectTip(e);
+    };
 }
 
 template<class Graph>
@@ -316,10 +323,10 @@ class LowCoverageEdgeRemovingAlgorithm : public PersistentProcessingAlgorithm<Gr
 
 protected:
 
-    void PrepareIteration(size_t it_cnt, size_t total_it_estimate) override {
-        TRACE("Preparing iteration " << it_cnt << " out of total estimate " << total_it_estimate);
+    void PrepareIteration(double iter_run_progress) override {
+        TRACE("Preparing iteration for iterative progress " << iter_run_progress);
         ConditionParser<Graph> parser(this->g(), condition_str_,
-                                      simplif_info_, it_cnt, total_it_estimate);
+                                      simplif_info_, iter_run_progress);
         remove_condition_ = omnigraph::AddAlternativesPresenceCondition(this->g(), parser());
         TRACE("Updated remove condition");
         proceed_condition_ = CoverageUpperBound<Graph>(this->g(), parser.max_coverage_bound());
@@ -347,21 +354,18 @@ public:
                                      const SimplifInfoContainer &simplif_info,
                                      std::function<void(EdgeId)> removal_handler = nullptr,
                                      bool canonical_only = true,
-                                     bool track_changes = true,
-                                     size_t total_iteration_estimate = -1ul)
+                                     bool track_changes = true)
             : base(g, nullptr,
                    canonical_only,
                    omnigraph::CoverageComparator<Graph>(g),
-                   track_changes,
-                   total_iteration_estimate),
+                   track_changes),
               simplif_info_(simplif_info),
               condition_str_(condition_str),
               edge_remover_(g, removal_handler),
               remove_condition_(func::AlwaysFalse<EdgeId>()),
               proceed_condition_(func::AlwaysTrue<EdgeId>()) {
 
-        ConditionParser<Graph> parser(g, condition_str, simplif_info,
-                                      total_iteration_estimate - 1, total_iteration_estimate);
+        ConditionParser<Graph> parser(g, condition_str, simplif_info);
         this->interest_el_finder_ =
                 std::make_shared<omnigraph::ParallelInterestingElementFinder<Graph>>(
                         AddAlternativesPresenceCondition(g, parser()),
@@ -394,6 +398,8 @@ template<class Graph>
 AlgoPtr<Graph> SelfConjugateEdgeRemoverInstance(Graph &g, const string &condition_str,
                 const SimplifInfoContainer &info,
                 EdgeRemovalHandlerF<Graph> removal_handler = 0) {
+    if (condition_str.empty())
+        return nullptr;
     ConditionParser<Graph> parser(g, condition_str, info);
     auto condition = func::And(SelfConjugateCondition<Graph>(g), parser());
 
@@ -447,10 +453,16 @@ AlgoPtr<Graph> RelativelyLowCoverageDisconnectorInstance(Graph &g,
         INFO("Disconnection of relatively low covered edges disabled");
         return nullptr;
     }
+    using Condition=omnigraph::simplification::relative_coverage::RelativeCovDisconnectionCondition<Graph>;
+
+    func::TypedPredicate<EdgeId> condition = Condition(g, flanking_cov, rced_config.diff_mult, rced_config.edge_sum);
+
+    if (math::gr(rced_config.unconditional_diff_mult, 0.)) {
+        condition = func::Or(Condition(g, flanking_cov, rced_config.unconditional_diff_mult, 0), condition);
+    }
 
     return std::make_shared<omnigraph::DisconnectionAlgorithm<Graph>>(g,
-            omnigraph::simplification::relative_coverage::
-            RelativeCovDisconnectionCondition<Graph>(g, flanking_cov, rced_config.diff_mult, rced_config.edge_sum),
+            condition,
             info.chunk_cnt(),
             nullptr);
 }
@@ -532,14 +544,12 @@ template<class Graph>
 AlgoPtr<Graph> ECRemoverInstance(Graph &g,
                                  const config::debruijn_config::simplification::erroneous_connections_remover &ec_config,
                                  const SimplifInfoContainer &info,
-                                 EdgeRemovalHandlerF<Graph> removal_handler = nullptr,
-                                 size_t iteration_cnt = 1) {
+                                 EdgeRemovalHandlerF<Graph> removal_handler = nullptr) {
     if (ec_config.condition.empty())
         return nullptr;
 
     return std::make_shared<LowCoverageEdgeRemovingAlgorithm<Graph>>(
-            g, ec_config.condition, info, removal_handler,
-            /*canonical only*/ true, /*track changes*/ true, iteration_cnt);
+            g, ec_config.condition, info, removal_handler);
 }
 
 template<class Graph>
@@ -567,7 +577,14 @@ AlgoPtr<Graph> TipClipperInstance(Graph &g,
 
     ConditionParser<Graph> parser(g, tc_config.condition, info);
     auto condition = parser();
-    return TipClipperInstance(g, condition, info, removal_handler, /*track changes*/true);
+    auto algo = TipClipperInstance(g, condition, info, removal_handler);
+    VERIFY_MSG(parser.requested_iterations() != 0, "To disable tip clipper pass empty string");
+    if (parser.requested_iterations() == 1) {
+        return algo;
+    } else {
+        return make_shared<LoopedAlgorithm<Graph>>(g, algo, 1, size_t(parser.requested_iterations()),
+                /*force primary for all*/ true);
+    }
 }
 
 template<class Graph>
@@ -679,18 +696,15 @@ template<class Graph>
 bool RemoveHiddenLoopEC(Graph &g,
                         const FlankingCoverage<Graph> &flanking_cov,
                         double determined_coverage_threshold,
-                        config::debruijn_config::simplification::hidden_ec_remover her_config,
+                        double relative_threshold,
                         EdgeRemovalHandlerF<Graph> removal_handler) {
-    if (her_config.enabled) {
-        INFO("Removing loops and rc loops with erroneous connections");
-        ECLoopRemover<Graph> hc(g, flanking_cov,
-                                determined_coverage_threshold,
-                                her_config.relative_threshold, removal_handler);
-        bool res = hc.Run();
-        hc.PrintLoopStats();
-        return res;
-    }
-    return false;
+    INFO("Removing loops and rc loops with erroneous connections");
+    ECLoopRemover<Graph> hc(g, flanking_cov,
+                            determined_coverage_threshold,
+                            relative_threshold, removal_handler);
+    bool res = hc.Run();
+    hc.PrintLoopStats();
+    return res;
 }
 
 }
