@@ -5,7 +5,7 @@ void create_console_logger() {
 
     string log_props_file = "log.properties";
 
-    logger *lg = create_logger(path::FileExists(log_props_file) ? log_props_file : "");
+    logger *lg = create_logger(fs::FileExists(log_props_file) ? log_props_file : "");
     lg->add_writer(std::make_shared<console_writer>());
     attach_logger(lg);
 }
