@@ -24,7 +24,7 @@ ALLOWED_READS_EXTENSIONS += [x + '.gz' for x in ALLOWED_READS_EXTENSIONS]
 
 # we support up to MAX_LIBS_NUMBER libs for each type of short-reads libs
 MAX_LIBS_NUMBER = 9
-OLD_STYLE_READS_OPTIONS = ["--12", "-1", "-2", "-s"]
+OLD_STYLE_READS_OPTIONS = ["--12", "-1", "-2", "-s", "--merged"]
 SHORT_READS_TYPES = {"pe": "paired-end", "s": "single", "mp": "mate-pairs", "hqmp": "hq-mate-pairs", "nxmate": "nxmate"}
 # other libs types:
 LONG_READS_TYPES = ["pacbio", "sanger", "nanopore", "tslr", "trusted-contigs", "untrusted-contigs"]
@@ -97,7 +97,8 @@ iterations = None
 bh_heap_check = None
 spades_heap_check = None
 read_buffer_size = None
-lcer_cutoff = None 
+lcer_cutoff = None
+read_cov_threshold = None
 ### END OF OPTIONS
 
 # for restarting SPAdes
@@ -136,14 +137,15 @@ dict_of_prefixes = dict()
 dict_of_rel2abs = dict()
 
 # list of spades.py options
-long_options = "12= threads= memory= tmp-dir= iterations= phred-offset= sc iontorrent meta large-genome rna plasmid "\
+long_options = "12= merged= threads= memory= tmp-dir= iterations= phred-offset= sc iontorrent meta large-genome rna plasmid "\
                "ss-fr ss-rf fast fast:false "\
                "only-error-correction only-assembler "\
                "disable-gzip-output disable-gzip-output:false disable-rr disable-rr:false " \
                "help version test debug debug:false reference= series-analysis= config-file= dataset= "\
                "bh-heap-check= spades-heap-check= read-buffer-size= help-hidden "\
                "mismatch-correction mismatch-correction:false careful careful:false save-gp save-gp:false "\
-               "continue restart-from= diploid truseq cov-cutoff= hidden-cov-cutoff= configs-dir= stop-after=".split()
+               "continue restart-from= diploid truseq cov-cutoff= hidden-cov-cutoff= read-cov-threshold= " \
+               "configs-dir= stop-after=".split()
 short_options = "o:1:2:s:k:t:m:i:hv"
 
 # adding multiple paired-end, mate-pair and other (long reads) libraries support
@@ -156,6 +158,8 @@ for i in range(MAX_LIBS_NUMBER):
             reads_options += ("%s%d-1= %s%d-2=" % tuple([type, i + 1] * 2)).split()
         else:  # paired-end, mate-pairs, hq-mate-pairs
             reads_options += ("%s%d-1= %s%d-2= %s%d-12= %s%d-s= %s%d-rf %s%d-fr %s%d-ff" % tuple([type, i + 1] * 7)).split()
+            if type == 'pe':  # special case: paired-end may include merged reads (-m)
+                reads_options += ["%s%d-m=" % (type, i+1)]
 reads_options += list(map(lambda x: x + '=', LONG_READS_TYPES))
 long_options += reads_options
 # for checking whether option corresponds to reads or not
@@ -216,18 +220,25 @@ def usage(spades_version, show_hidden=False, mode=None):
     sys.stderr.write("-1\t<filename>\tfile with forward paired-end reads" + "\n")
     sys.stderr.write("-2\t<filename>\tfile with reverse paired-end reads" + "\n")
     sys.stderr.write("-s\t<filename>\tfile with unpaired reads" + "\n")
+    sys.stderr.write("--merged\t<filename>\tfile with merged forward and reverse paired-end reads" + "\n")
+    if mode == "meta":
+        allowed_lib_ids = "1"
+    else:
+        allowed_lib_ids = "1,2,...," + str(MAX_LIBS_NUMBER)
     sys.stderr.write("--pe<#>-12\t<filename>\tfile with interlaced"\
-                         " reads for paired-end library number <#> (<#> = 1,2,..,9)" + "\n")
+                         " reads for paired-end library number <#> (<#> = %s)" % allowed_lib_ids + "\n")
     sys.stderr.write("--pe<#>-1\t<filename>\tfile with forward reads"\
-                         " for paired-end library number <#> (<#> = 1,2,..,9)" + "\n")
+                         " for paired-end library number <#> (<#> = %s)" % allowed_lib_ids + "\n")
     sys.stderr.write("--pe<#>-2\t<filename>\tfile with reverse reads"\
-                         " for paired-end library number <#> (<#> = 1,2,..,9)" + "\n")
+                         " for paired-end library number <#> (<#> = %s)" % allowed_lib_ids + "\n")
     sys.stderr.write("--pe<#>-s\t<filename>\tfile with unpaired reads"\
-                         " for paired-end library number <#> (<#> = 1,2,..,9)" + "\n")
+                         " for paired-end library number <#> (<#> = %s)" % allowed_lib_ids + "\n")
+    sys.stderr.write("--pe<#>-m\t<filename>\tfile with merged reads"\
+                         " for paired-end library number <#> (<#> = %s)" % allowed_lib_ids + "\n")
     sys.stderr.write("--pe<#>-<or>\torientation of reads"\
-                         " for paired-end library number <#> (<#> = 1,2,..,9; <or> = fr, rf, ff)" + "\n")
+                         " for paired-end library number <#> (<#> = %s; <or> = fr, rf, ff)" % allowed_lib_ids + "\n")
     sys.stderr.write("--s<#>\t\t<filename>\tfile with unpaired reads"\
-                     " for single reads library number <#> (<#> = 1,2,..,9)" + "\n")
+                     " for single reads library number <#> (<#> = %s)" % allowed_lib_ids + "\n")
     if mode not in ["rna", "meta"]:
         sys.stderr.write("--mp<#>-12\t<filename>\tfile with interlaced"\
                              " reads for mate-pair library number <#> (<#> = 1,2,..,9)" + "\n")
@@ -254,12 +265,13 @@ def usage(spades_version, show_hidden=False, mode=None):
         sys.stderr.write("--nxmate<#>-2\t<filename>\tfile with reverse reads"\
                              " for Lucigen NxMate library number <#> (<#> = 1,2,..,9)" + "\n")
         sys.stderr.write("--sanger\t<filename>\tfile with Sanger reads\n")
+    if not mode == "rna":
         sys.stderr.write("--pacbio\t<filename>\tfile with PacBio reads\n")
         sys.stderr.write("--nanopore\t<filename>\tfile with Nanopore reads\n")
-    if not mode == "rna":
         sys.stderr.write("--tslr\t<filename>\tfile with TSLR-contigs\n")
-    sys.stderr.write("--trusted-contigs\t<filename>\tfile with trusted contigs\n")
-    sys.stderr.write("--untrusted-contigs\t<filename>\tfile with untrusted contigs\n")
+    if not mode == "meta":
+        sys.stderr.write("--trusted-contigs\t<filename>\tfile with trusted contigs\n")
+        sys.stderr.write("--untrusted-contigs\t<filename>\tfile with untrusted contigs\n")
     if mode == "dip":
         sys.stderr.write("Input haplocontigs:" + "\n")
         sys.stderr.write("--hap\t<filename>\tfile with haplocontigs" + "\n")
@@ -277,7 +289,10 @@ def usage(spades_version, show_hidden=False, mode=None):
         if mode not in ["rna", "meta"]:
             sys.stderr.write("--careful\t\ttries to reduce number of mismatches and short indels" + "\n")
         sys.stderr.write("--continue\t\tcontinue run from the last available check-point" + "\n")
-        sys.stderr.write("--restart-from\t<cp>\trestart run with updated options and from the specified check-point ('ec', 'as', 'k<int>', 'mc')" + "\n")
+        if mode == "rna":
+            sys.stderr.write("--restart-from\t<cp>\trestart run with updated options and from the specified check-point ('ec', 'as', 'last')" + "\n")
+        else:
+            sys.stderr.write("--restart-from\t<cp>\trestart run with updated options and from the specified check-point ('ec', 'as', 'k<int>', 'mc', 'last')" + "\n")
     sys.stderr.write("--disable-gzip-output\tforces error correction not to"\
                          " compress the corrected reads" + "\n")
     sys.stderr.write("--disable-rr\t\tdisables repeat resolution stage"\
@@ -340,6 +355,7 @@ def usage(spades_version, show_hidden=False, mode=None):
         sys.stderr.write("--save-gp\tEnables saving graph pack before repeat resolution (even without --debug) \n")
         sys.stderr.write("--hidden-cov-cutoff\t<float>\t\tcoverage cutoff value deeply integrated in simplification"\
                             " (a positive float number). Base coverage! Will be adjusted depending on K and RL! \n")
+        sys.stderr.write("--read-cov-threshold\t<int>\t\tread median coverage threshold (non-negative integer)\n")
         sys.stderr.write("--help-hidden\tprints this usage message with all hidden options" + "\n")
 
     if show_hidden and mode == "dip":
