@@ -63,10 +63,6 @@ def prepare_config_spades(filename, cfg, log, additional_contigs_fname, K, stage
         subst_dict["lcer_enabled"] = bool_to_str(True)
         subst_dict["lcer_coverage_threshold"] = cfg.lcer_cutoff
 
-    #TODO: make something about spades.py and config param substitution 
-    if "bwa_paired" in cfg.__dict__:
-        subst_dict["bwa_enable"] = bool_to_str(True)
-    subst_dict["path_to_bwa"] =  os.path.join(execution_home, "bwa-spades")
     if "series_analysis" in cfg.__dict__:
         subst_dict["series_analysis"] = cfg.series_analysis
     process_cfg.substitute_params(filename, subst_dict, log)
@@ -81,6 +77,14 @@ def prepare_config_rnaspades(filename, log):
     process_cfg.substitute_params(filename, subst_dict, log)
 
 
+def prepare_config_construction(filename, log):
+    if options_storage.read_cov_threshold is None:
+        return
+    subst_dict = dict()
+    subst_dict["read_cov_threshold"] = options_storage.read_cov_threshold
+    process_cfg.substitute_params(filename, subst_dict, log)
+
+
 def get_read_length(output_dir, K, ext_python_modules_home, log):
     est_params_filename = os.path.join(output_dir, "K%d" % K, "final.lib_data")
     max_read_length = 0
@@ -91,10 +95,8 @@ def get_read_length(output_dir, K, ext_python_modules_home, log):
         elif sys.version.startswith('3.'):
             import pyyaml3 as pyyaml
         est_params_data = pyyaml.load(open(est_params_filename, 'r'))
-        for reads_library in est_params_data:
-            if reads_library['type'] in READS_TYPES_USED_IN_CONSTRUCTION:
-                if int(reads_library["read length"]) > max_read_length:
-                    max_read_length = int(reads_library["read length"])
+        max_read_length = int(est_params_data['nomerge max read length'])
+        log.info("Max read length detected as %d" % max_read_length)
     if max_read_length == 0:
         support.error("Failed to estimate maximum read length! File with estimated params: " + est_params_filename, log)
     return max_read_length
@@ -202,10 +204,11 @@ def run_iteration(configs_dir, execution_home, cfg, log, K, prev_K, last_one):
         process_cfg.substitute_params(os.path.join(dst_configs, "pe_params.info"), {"scaffolding_mode": cfg.scaffolding_mode}, log)
 
     prepare_config_rnaspades(os.path.join(dst_configs, "rna_mode.info"), log)
+    prepare_config_construction(os.path.join(dst_configs, "construction.info"), log)
     cfg_fn = os.path.join(dst_configs, "config.info")
     prepare_config_spades(cfg_fn, cfg, log, additional_contigs_fname, K, stage, saves_dir, last_one, execution_home)
 
-    command = [os.path.join(execution_home, "spades"), cfg_fn]
+    command = [os.path.join(execution_home, "spades-core"), cfg_fn]
 
     add_configs(command, dst_configs)
 
@@ -253,7 +256,7 @@ def run_scaffold_correction(configs_dir, execution_home, cfg, log, latest, K):
         process_cfg.substitute_params(construction_cfg_file_name, {"read_buffer_size": cfg.read_buffer_size}, log)
     process_cfg.substitute_params(os.path.join(dst_configs, "moleculo_mode.info"), {"scaffolds_file": scaffolds_file}, log)
     prepare_config_scaffold_correction(cfg_file_name, cfg, log, saves_dir, K)
-    command = [os.path.join(execution_home, "scaffold_correction"), cfg_file_name]
+    command = [os.path.join(execution_home, "spades-truseq-scfcorrection"), cfg_file_name]
     add_configs(command, dst_configs)
     log.info(str(command))
     support.sys_call(command, log)
